/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

 
#ifndef __TIZEN_SOCIAL_CALENDAR_CALENDAR_H__
#define __TIZEN_SOCIAL_CALENDAR_CALENDAR_H__

#include <tizen.h>
#include <time.h>
#include <calendar_attendee.h>
#include <calendar_types.h>


#ifdef __cplusplus
extern "C" {
#endif

/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_MODULE
 * @{
 */

/**
 * @brief Connects to the calendar service.
 *
 * @details Opening connection is necessary to access the calendar database. 
 * All operations like inserting, updating, or deleting calendar require opened connection to work properly.

 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE Successful
 * @retval  #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @see calendar_disconnect()
 *
 */
int calendar_connect(void);

/**
 * @brief Disconnects the calendar service.
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE Successful
 * @retval  #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @see calendar_connect()
 *
 */
int calendar_disconnect(void);

/**
 * @brief  Retrieves all timezone information supported on a device.
 *
 * @param[in]   callback    The callback function to invoke 
 * @param[in]   user_data   The user data to be passed to the callback function
 *
 * @return   0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @post	This function invokes calendar_foreach_timezone_cb().
 *
 * @see calendar_foreach_timezone_cb()
 * @see calendar_event_set_timezone()
 * @see calendar_todo_set_timezone()
 */
int calendar_foreach_timezone(calendar_foreach_timezone_cb callback, void *user_data); 

/**
 * @brief  Registers a callback function to be invoked when any database change occur.
 *
 * @param[in]   callback    The callback function to invoke 
 * @param[in]   user_data    The user data to be passed to the callback function
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 * @post  calendar_db_changed_cb() will be invoked.
 * 
 * @see calendar_db_changed_cb()
 * @see calendar_unset_db_changed_cb()
 * @see	calendar_event_insert_to_db()
 * @see	calendar_event_update_to_db()
 * @see	calendar_event_delete_from_db()
 */
int calendar_set_db_changed_cb(calendar_db_changed_cb callback, void *user_data);

/**
 * @brief	Unregisters the callback function.
 *
 * @param[in]	callback	The callback function to unregister
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 * @see calendar_db_changed_cb()
 * @see  calendar_set_db_changed_cb()
 */
int calendar_unset_db_changed_cb(calendar_db_changed_cb callback);

/**
 * @}
 */

/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_EVENT_MODULE
 * @{
 */


/**
 * @brief Creates a handle to the calendar event.
 * @remarks The calendar event handle must be released with calendar_event_destroy() by you.
 * @remarks   The created handle is not added to calendar database until calendar_event_insert_to_db() is called
 *
 * @param[out] event A new handle to calendar event
 *
 * @return   0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY       Out of memory
 *
 * @see calendar_event_destroy()
 *
 */
int calendar_event_create(calendar_event_h *event);

/**
 * @brief Destroys the calendar event handle and releases all its resources.
 *
 * @param[out]  event   The calendar event handle 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_create()
 *
 */
int calendar_event_destroy(calendar_event_h event);

/**
 * @brief Adds a calendar event to the calendar database.
 *
 * @param[in]	event               The calendar event handle 
 * @param[in]   calendar_db_id      The calendar database ID to which the event insert. \n
 *                       1 means the default calendar on the device
 * @param[out]	event_db_id         The event db ID associated with the calendar event \n
 * If the function fails, it is -1.
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @see calendar_connect()
 * @see calendar_event_update_to_db()
 * @see calendar_event_delete_from_db()
 */
int calendar_event_insert_to_db(calendar_event_h event, int calendar_db_id, int *event_db_id);

/**
 * @brief Removes the calendar event from the calendar database.
 *
 * @param[in]   event_id  The event ID to delete
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @pre	This function requires an open connection to the calendar service by calendar_connect().
 *
 * @see calendar_connect()
 * @see	calendar_event_insert_to_db()
 * @see	calendar_event_update_to_db()
 */
int calendar_event_delete_from_db(int event_id);

/**
 * @brief Updates the calendar event in the calendar database.
 * @details calendar_event_get_db_id() is called internally to update in the calendar database to get the event ID in the calendar database.
 *
 * @param[in]   event The calendar event handle 
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE					Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER	Invalid parameter
 * @retval #CALENDAR_ERROR_DB_FAILED 			No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 * @pre		The event is supposed to be in the calendar database already.
 *
 * @see calendar_connect()
 * @see calendar_event_insert_to_db()
 * @see calendar_event_delete_from_db()
 * @see calendar_event_get_from_db()
 * @see calendar_event_get_db_id()
 */
int calendar_event_update_to_db(calendar_event_h event);

/**
 * @brief Gets the calendar event from the calendar database.
 * @details This function gets the new calendar event handle with the given event database ID from the calendar database. \n
 * The new calendar event handle will be created. 
 *
 * @remarks The created calendar event handle must be released with calendar_event_destroy() by you. 
 *
 *
 * @param[in]   event_id	The event ID
 * @param[out]  event	The calendar event handle associated with the event database ID
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE  Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY       Out of memory
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @see calendar_event_destroy()
 */
int calendar_event_get_from_db(int event_id, calendar_event_h *event);

/**
 * @brief   Gets the database ID associated with the given calendar event handle. 
 *
 * @param[in]   event	The calendar event handle
 * @param[out]  event_db_id	The event database ID fetched from the calendar event handle (default : 0)  \n
 * 0 means the event is not in calendar database. 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_insert_to_db()
 * @see calendar_event_delete_from_db()
 */
int calendar_event_get_db_id(calendar_event_h event, int *event_db_id);

/**
 * @brief   Gets the calendar database ID associated with the given calendar event handle. 
 *
 * @param[in]   event			The calendar event handle
 * @param[out]  calendar_db_id	The calendar database ID fetched from the calendar event handle (default : 0)  \n
 * 0 means the event is not in calendar database. 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_insert_to_db()
 */
int calendar_event_get_calendar_db_id(calendar_event_h event, int *calendar_db_id);

/**
 * @brief Gets the subject of the event from the calendar event handle .
 *
 * @remarks @a subject must be released with free() by you.
 *
 * @param [in] event The calendar event handle 
 * @param [out] subject The subject of the event  \n If the subject does not exist, it is NULL
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_set_subject()
 *
*/
int calendar_event_get_subject(calendar_event_h event, char **subject);


/**
 * @brief Sets the subject of the event in the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [in] subject The subject of the event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_get_subject()
 */
int calendar_event_set_subject(calendar_event_h event, const char *subject);


/**
 * @brief Gets the description of the event from the calendar event handle.
 *
 * @remarks @a description must be released with free() by you. 
 *
 * @param [in] event The calendar event handle 
 * @param [out] description The description of the event \n If the description does not exist, it is NULL
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE					Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER	Invalid parameter
 *
 * @see calendar_event_set_description()
 */
int calendar_event_get_description(calendar_event_h event, char **description);


/**
 * @brief Sets a description of the event in the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [in] description The description of the event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_get_description()
 */
int calendar_event_set_description(calendar_event_h event, const char *description);


/**
 * @brief Gets an all-day event property of the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [out] is_all_day_event @c true if the event lasts for an entire day, otherwise @c false \n
 * (default : @c false)
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_set_is_all_day_event()
 */
int calendar_event_get_is_all_day_event(calendar_event_h event, bool *is_all_day_event);


/**
 * @brief Sets an all-day event property of the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [in] is_all_day_event The all-day event type \n
 *		@c true if the event will last for an entire day, otherwise @c false \n
 * (default : @c false)
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_get_is_all_day_event()
 */
int calendar_event_set_is_all_day_event(calendar_event_h event, bool is_all_day_event);


/**
 * @brief Gets the location of the event from the calendar event handle.
 *
 * @remarks @a location must be released with free() by you. 
 *
 * @param [in] event The calendar event handle 
 * @param [out] location The location of the event \n If the location does not exist, it is NULL
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_set_location()
 */
int calendar_event_get_location(calendar_event_h event, char **location);


/**
 * @brief Sets a location of the event in the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [in] location The location of the event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_get_location()
 */
int calendar_event_set_location(calendar_event_h event, const char *location);


/**
 * @brief Gets the reminder of the event from the calendar event handle.
 * @details The function returns reminder type and interval of the event.
 *  
 *
 * @param [in] event The calendar event handle 
 * @param [out] reminder_interval_type  The reminder type 
 * @param [out] reminder_interval       The interval  
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_set_reminder()
 */
int calendar_event_get_reminder(calendar_event_h event, calendar_reminder_interval_type_e *reminder_interval_type, int *reminder_interval);


/**
 * @brief Sets a reminder of the event in the calendar event handle.
 *
 * @remarks If reminder is set, the alarm will remind the event with in-house calendar application.
 *
 * @param [in] event The calendar event handle 
 * @param [in] reminder_interval_type  The reminder interval type
 * @param [in] reminder_interval  The reminder interval
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE                 Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER    Invalid parameter
 *
 * @see calendar_event_get_reminder()
 */
int calendar_event_set_reminder(calendar_event_h event, calendar_reminder_interval_type_e reminder_interval_type, int reminder_interval);

/**
 * @brief Gets the recurrence frequency of the event from the calendar event handle. 
 *
 * @param[in]   event The calendar event handle 
 * @param[out]  recurrence_frequency    The recurrence type of the event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_set_recurrence_frequency()
 * @see calendar_event_set_recurrence_week_flag()
 * @see calendar_event_get_recurrence_week_flag()
 * @see calendar_event_add_recurrence_exception_date()
 */
int calendar_event_get_recurrence_frequency(calendar_event_h event, calendar_recurrence_frequency_e *recurrence_frequency);


/**
 * @brief Sets a recurrence frequency of the event in the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [in] recurrence_frequency The recurrence frequency of the event \n 
 * If you set it to #CALENDAR_RECURRENCE_WEEKLY, you have to set calendar_event_set_recurrence_week_flag().
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_get_recurrence_frequency()
 * @see calendar_event_set_recurrence_week_flag()
 * @see calendar_event_get_recurrence_week_flag()
 * @see calendar_event_add_recurrence_exception_date()
 */
int calendar_event_set_recurrence_frequency(calendar_event_h event, calendar_recurrence_frequency_e recurrence_frequency);

/**
 * @brief Gets the recurrence frequency of the event in the calendar event handle.
 *
 * @remarks This function is valid only if recurrence frequency of the event is #CALENDAR_RECURRENCE_WEEKLY.
 *
 * @param[in]   event The calendar event handle 
 * @param[out]  week_flag    The days of week flag of the event \n It supports "OR"ing combination of #calendar_week_flag_e. \n eg) #CALENDAR_WEEK_FLAG_TUE | #CALENDAR_WEEK_FLAG_FRI
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_set_recurrence_week_flag()
 * @see calendar_event_set_recurrence_frequency()
 * @see calendar_event_get_recurrence_frequency()
 */
int calendar_event_get_recurrence_week_flag(calendar_event_h event, int *week_flag);

/**
 * @brief   Sets the days of week flag to the calendar event handle. 
 *
 * @remarks This function is valid only if recurrence frequency of the event is #CALENDAR_RECURRENCE_WEEKLY.
 *
 * @param[in]   event The calendar event handle 
 * @param[in]   week_flag    The days of week flag of the event \n It supports "OR"ing combination of #calendar_week_flag_e. \n eg) #CALENDAR_WEEK_FLAG_TUE | #CALENDAR_WEEK_FLAG_FRI
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_get_recurrence_week_flag()
 * @see calendar_event_set_recurrence_frequency()
 * @see calendar_event_get_recurrence_frequency()
 *
 */
int calendar_event_set_recurrence_week_flag(calendar_event_h event, int week_flag);

/**
 * @brief Gets the recurrence ending time of the event from the calendar event handle.
 * @details  If recurrence frequency of the event is activated, then @a recurrence_until_date indicates recurrence ending time.
 *
 * @param [in] event The calendar event handle 
 * @param [out] recurrence_until_date The ending time of the recurrence event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #CALENDAR_ERROR_NO_DATA				Data not found
 *
 * @see calendar_event_set_recurrence_until_date()
 */
int calendar_event_get_recurrence_until_date(calendar_event_h event, struct tm *recurrence_until_date);


/**
 * @brief Sets a recurrence end time of the event in the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [in] recurrence_until_date  The end time of the recurrence event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @pre @a event has a recurrence frequency property using calendar_event_set_recurrence_frequency().
 * 
 * @see calendar_event_get_recurrence_until_date()
 */
int calendar_event_set_recurrence_until_date(calendar_event_h event, struct tm recurrence_until_date);

/**
 * @brief   Adds a recurrence exception date to the calendar event handle.
 *
 * @param[in]   event	 The calendar event handle 
 * @param[in]   year	 The year of date \n It has to be more than 1900
 * @param[in]   month	 The month of date [1-12] 
 * @param[in]   day		 The day of date [1-31] 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE				Successful
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY		Out of memory
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER	Invalid parameter
 *
 * @see calendar_event_set_recurrence_frequency()
 * @see calendar_event_get_recurrence_exception_iterator()
 */
int calendar_event_add_recurrence_exception_date(calendar_event_h event, int year, int month, int day);


/**
 * @brief Gets the recurrence exception iterator handle from the event handle.  
 *
 * @param[in]   event      The calendar event handle 
 * @param[out]  iterator    The recurrence exception iterator handle \n If the iterator does not exist, it is NULL
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_recurrence_exception_has_next()
 * @see calendar_recurrence_exception_next()
 */
int calendar_event_get_recurrence_exception_iterator(calendar_event_h event, calendar_recurrence_exception_iterator_h *iterator);

/**
 * @brief  Checks whether there is a next recurrence exception handle on the list. 
 *
 * @param [in] iterator  The recurrence exception iterator handle 
 *
 * @return  @c true If the next element exists or @c false If the next element doesn't exist 
 *
 * @see calendar_event_get_recurrence_exception_iterator()
 * @see calendar_recurrence_exception_next()
 */
bool calendar_recurrence_exception_has_next(calendar_recurrence_exception_iterator_h iterator);

/**
 * @brief  Moves recurrence exception list iterator to the next position and gets recurrence exception date. 
 * @details If the next element for the current recurrence exception list exists, then the iterator is moved to 
 * the next position on the list and the recurrence exception handle for this position is returned. 
 * When the iterator reaches the last element of the list, all further calls will return 
 * #CALENDAR_ERROR_ITERATOR_END and @a recurrence exception will remain unchanged.
 *
 * @param [in] iterator  The recurrent exception iterator handle 
 * @param [out]   year	 The year of date \n It is supposed to be more than 1900
 * @param [out]   month	 The month of date [1-12] 
 * @param [out]   day	 The day of date [1-31] 
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER    Invalid parameter
 * @retval #CALENDAR_ERROR_ITERATOR_END     Iterator is on the last position
 *
 * @see calendar_event_get_recurrence_exception_iterator()
 * @see calendar_recurrence_exception_has_next()
 */
int calendar_recurrence_exception_next(calendar_recurrence_exception_iterator_h *iterator, int* year, int* month, int* day);

/**
 * @brief Gets the start time of the event from the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [out] start_time The start date of the event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #CALENDAR_ERROR_NO_DATA				Data not found
 *
 * @see calendar_event_set_start_time()
 */
int calendar_event_get_start_time(calendar_event_h event, struct tm *start_time);


/**
 * @brief Sets a start time of the event in the calendar event handle. 
 *
 * @param [in] event The calendar event handle 
 * @param [in] start_time The start date of the event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_get_start_time()
 */
int calendar_event_set_start_time(calendar_event_h event, struct tm start_time);


/**
 * @brief Gets the end time of the event from the calendar event handle.
 *
 * @param [in] event The calendar event handle 
 * @param [out] end_time The end date of the event
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #CALENDAR_ERROR_NO_DATA				Data not found
 *
 * @see calendar_event_set_end_time()
 */
int calendar_event_get_end_time(calendar_event_h event, struct tm *end_time);


/**
 * @brief Sets the end time of the event in the calendar event handle.
 *
 * @param[in]   event The calendar event handle 
 * @param[in]   end_time    The end date of the event
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_get_end_time()
 */
int calendar_event_set_end_time(calendar_event_h event, struct tm end_time);

/**
 * @brief Gets the total number of events from the calendar database. 
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @param[out]  count   The total number of events
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @see calendar_connect()
 */
int calendar_event_get_total_count_from_db(int *count);

/**
 * @brief   Creates a attendee handle and make adds it to the calendar event.
 *
 * @remarks   The created handle is not added to calendar database until calendar_event_insert_to_db() is called
 *
 * @param[in]   event The calendar event handle 
 * @param[out]  attendee A new attendee handle
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE            Successful
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY   Out of memory
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_remove_attendee()
 * @see calendar_attendee_set_name()
 * @see calendar_attendee_set_email()
 * @see calendar_attendee_set_phone_number()
 * @see calendar_event_get_attendee_iterator()
 */
int calendar_event_add_attendee(calendar_event_h event, calendar_attendee_h* attendee);

/**
 * @brief   Creates a attendee handle with contact ID and adds it to the calendar event.
 *
 * @details Adds an existing contact as an attendee to an event.  A new attendee record is created, and the name, email address and phone number are populated from the values in the contact record.  
 * @remarks   The created handle is not added to calendar database until calendar_event_insert_to_db() is called
 *
 *
 * @param[in]   event The calendar event handle 
 * @param[in]   contact_db_id The contact DB ID which is related to attendee 
 * @param[out]  attendee A new attendee handle from contact ID in the contacts database
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE            Successful
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY   Out of memory
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @pre     This function requires an open connection to Contacts Service by contacts_connect(). 
 *
 * @see calendar_event_remove_attendee()
 * @see calendar_attendee_set_name()
 * @see calendar_attendee_set_email()
 * @see calendar_attendee_set_phone_number()
 * @see calendar_event_get_attendee_iterator()
 * @see contact_get_db_id()
 * @see contact_get_from_db()
 * @see calendar_attendee_get_phone_number()
 */
int calendar_event_add_attendee_with_contact(calendar_event_h event, int contact_db_id, calendar_attendee_h* attendee);

/**
 * @brief Removes the attendee from the calendar event handle.
 *
 *
 * @param[in]   event The calendar event handle 
 * @param[in]   attendee The attendee handle
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_event_add_attendee()
 * @see calendar_event_get_attendee_iterator()
 */
int calendar_event_remove_attendee(calendar_event_h event, calendar_attendee_h attendee);

/**
 * @brief Gets the attendee iterator handle from the event handle.  
 *
 * @param[in]   event      The calendar event handle 
 * @param[out]  iterator    The attendee iterator handle
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_attendee_iterator_has_next()
 * @see calendar_attendee_iterator_next()
 *
 */
int calendar_event_get_attendee_iterator(calendar_event_h event, calendar_attendee_iterator_h *iterator);

/**
 * @brief	Gets the last modified time of the event handle.
 *
 * @param[in]   event       The calendar event handle 
 * @param[out]  modified_time The last modified time of event 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE				Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER	Invalid parameter
 * @retval  #CALENDAR_ERROR_NO_DATA             Data not found
 *
 * @see calendar_event_insert_to_db()
 * @see calendar_event_update_to_db()
 */
int calendar_event_get_last_modified_time(calendar_event_h event, struct tm *modified_time); 

/**
 * @brief   Gets the visibility for the given calendar event handle. 
 *
 * @param[in]   event   		The calendar event handle
 * @param[out]  visibility	The visibility of the event \n (default : #CALENDAR_VISIBILITY_PUBLIC)
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_set_visibility()
 */
int calendar_event_get_visibility(calendar_event_h event, calendar_visibility_e *visibility);

/**
 * @brief   Sets a visibility for the given calendar event handle. 
 *
 * @param[in]   event   		The calendar event handle
 * @param[in]	visibility		The visibility of the event
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_get_visibility()
 */
int calendar_event_set_visibility(calendar_event_h event, calendar_visibility_e visibility);

/**
 * @brief   Gets the timezone details for the given calendar event handle. 
 *
 * @remark @a timezone_name must be released with free() by you.
 *
 * @param[in]   event   					The calendar event handle
 * @param[out]  timezone_name				The time zone name
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_foreach_timezone()
 * @see calendar_event_set_timezone()
 */
int calendar_event_get_timezone(calendar_event_h event, char** timezone_name);

/**
 * @brief   Sets a timezone for the given calendar event handle. 
 *
 * @param[in]   event   					The calendar event handle
 * @param[in]  timezone_name			The time zone name 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_foreach_timezone()
 * @see calendar_event_get_timezone()
 */
int calendar_event_set_timezone(calendar_event_h event, const char* timezone_name);

/**
 * @brief   Gets an event from vCalendar and returns a new event handle. 
 *
 * @remarks The new event handle must be released with calendar_event_destroy() by you. 
 *
 * @param[in]   vcalendar_stream        The vCalendar stream to convert
 * @param[out]  event             The event handle
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY       Out of memory
 *
 * @see calendar_event_destroy()
 * @see calendar_event_get_vcalendar_from_event()
 */
int calendar_event_get_from_vcalendar(const char *vcalendar_stream, calendar_event_h *event);

/**
 * @brief   Gets vCalendar from the event handle. 
 * 
 * @remarks @a vcalendar_stream must be released with free() by you. 
 *
 * @param[in]   event     		   The event handle
 * @param[out]  vcalendar_stream           The vCalendar stream to convert
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_event_get_from_vcalendar()
 */
int calendar_event_get_vcalendar_from_event(calendar_event_h event, char **vcalendar_stream);

/**
 * @brief  Retrieves all calendar events from all of calendars by invoking the given callback function iteratively. 
 *
 * @param[in]   callback    The callback function to invoke
 * @param[in]   user_data   The user data to be passed to the callback function
 *
 * @return   0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_event_cb().
 *
 * @see calendar_foreach_query_event_cb()
 */
int calendar_foreach_event_from_db(calendar_foreach_query_event_cb callback, void *user_data);

/**
 * @brief  Retrieves all calendar events with the given calendar database ID.
 *
 * @param[in]   callback        The callback function to invoke
 * @param[in]   calendar_db_id      The calendar database ID to filter \n
 *                       1 means the default calendar on the device
 * @param[in]   user_data        The user data to be passed to the callback function
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_DB_FAILED           No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_event_cb(). 
 *
 * @see calendar_foreach_query_event_cb()
 * @see calendar_foreach_event_from_db()
 * @see calendar_query_event_by_description()
 * @see calendar_query_event_by_location()
 * @see calendar_query_event_by_period()
 */
int calendar_query_event_by_calendar(calendar_foreach_query_event_cb callback, int calendar_db_id, void *user_data);

/**
 * @brief  Retrieves all calendar events with the given subject.
 *
 * @param[in]   callback        The callback function to invoke
 * @param[in]   calendar_db_id      The calendar database ID to filter \n
 *                       1 means the default calendar on the device
 * @param[in]   subject_to_find The description to filter
 * @param[in]   user_data        The user data to be passed to the callback function
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_DB_FAILED           No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_event_cb(). 
 *
 * @see calendar_event_get_subject()
 * @see calendar_event_set_subject()
 * @see calendar_foreach_query_event_cb()
 * @see calendar_query_event_by_calendar()
 * @see calendar_query_event_by_description()
 * @see calendar_query_event_by_location()
 * @see calendar_query_event_by_period()
 */
int calendar_query_event_by_subject(calendar_foreach_query_event_cb callback, int calendar_db_id, const char *subject_to_find, void *user_data);

/**
 * @brief   Retrieves all calendar events with the given description.
 *
 * @param[in]   callback            The callback function to invoke
 * @param[in]   calendar_db_id      The calendar database ID to filter \n
 *                       1 means the default calendar on the device
 * @param[in]   description_to_find The description to filter
 * @param[in]   user_data           The user data to be passed to the callback function
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_DB_FAILED           No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post 	This function invokes calendar_foreach_query_event_cb().
 *
 * @see   calendar_event_get_description()
 * @see   calendar_event_set_description()
 * @see calendar_foreach_query_event_cb()
 * @see calendar_query_event_by_calendar()
 * @see   calendar_query_event_by_subject()
 * @see   calendar_query_event_by_location()
 * @see   calendar_query_event_by_period()
 * @see   calendar_foreach_query_event_cb()
 */
int calendar_query_event_by_description(calendar_foreach_query_event_cb callback, int calendar_db_id, const char *description_to_find, void *user_data);

/**
 * @brief   Retrieves all calendar events with the given location.
 *
 * @param[in]   callback               The callback function to invoke
 * @param[in]   calendar_db_id      The calendar database ID to filter \n
 *                       1 means the default calendar on the device
 * @param[in]   location_to_find    The location to filter
 * @param[in]   user_data            The user data to be passed to the callback function
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_DB_FAILED           No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_event_cb().
 *
 * @see calendar_event_get_location()
 * @see calendar_event_set_location()
 * @see calendar_foreach_query_event_cb()
 * @see calendar_query_event_by_calendar()
 * @see calendar_query_event_by_subject()
 * @see calendar_query_event_by_description()
 * @see calendar_query_event_by_period()
 */
int calendar_query_event_by_location(calendar_foreach_query_event_cb callback, int calendar_db_id, const char *location_to_find, void *user_data);

/**
 * @brief   Retrieves all calendar events between the given start_time and end_time.  
 *
 * @param[in]   callback    The callback function to invoke
 * @param[in]   calendar_db_id      The calendar database ID to filter \n
 *                       1 means the default calendar on the device
 * @param[in]   start_time  The start date
 * @param[in]   end_time    The end date
 * @param[in]   user_data    The user data to be passed to the callback function
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_DB_FAILED           No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_event_cb().
 *
 * @see calendar_event_get_start_time()
 * @see calendar_event_set_start_time()
 * @see calendar_event_get_end_time()
 * @see calendar_event_set_end_time()
 * @see calendar_foreach_query_event_cb()
 * @see calendar_query_event_by_calendar()
 * @see calendar_query_event_by_subject()
 * @see calendar_query_event_by_description()
 * @see calendar_query_event_by_location()
 * @see calendar_foreach_query_event_cb()
 *
 */
int calendar_query_event_by_period(calendar_foreach_query_event_cb callback, int calendar_db_id, struct tm start_time, struct tm end_time, void *user_data);

/**
 * @brief   Retrieves all calendar events that have been modified since the given time.
 * @details This function will find all changed event since the given time
 *
 * @param[in]   callback             The callback function to invoke
 * @param[in]   calendar_db_id      The calendar database ID to filter \n
 *                       1 means the default calendar on the device
 * @param[in]   time 				 The time to find events which are changed since when
 * @param[in]   user_data            The user data to be passed to the callback function
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_DB_FAILED           No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_event_cb().
 *
 * @see calendar_event_get_last_modified_time()
 * @see calendar_foreach_query_event_cb()
 */
int calendar_query_event_by_time_last_modified(calendar_foreach_query_event_cb callback, int calendar_db_id, struct tm time, void *user_data);

/**
 * @}
 */



/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_TODO_MODULE
 * @{
 */

/**
 * @brief Creates a handle to calendar to-do.
 * @remarks The calendar to-do handle must be released with calendar_todo_destroy() by you.
 * @remarks   The created handle is not added to calendar database until calendar_todo_insert_to_db() is called
 *
 * @param[out] todo		The new calendar to-do handle
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY       Out of memory
 *
 * @see calendar_todo_destroy()
 *
 */
int calendar_todo_create(calendar_todo_h *todo);

/**
 * @brief Destroys the calendar to-do handle 
 *
 * @param[in]  todo   The calendar to-do handle 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_create()
 *
 */
int calendar_todo_destroy(calendar_todo_h todo);

/**
 * @brief Adds the given to-do item to the calendar database.
 *
 * @param[in]   todo		The calendar to-do handle 
 * @param[in]   calendar_db_id      The calendar database ID to which the event insert. \n
 *                       1 means the default calendar on the device
 * @param[out]  todo_db_id		The to-do ID to be assigned to the to-do item
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 * @see calendar_todo_delete_from_db()
 * @see calendar_todo_update_to_db()
 *
 */
int calendar_todo_insert_to_db(calendar_todo_h todo, int calendar_db_id, int *todo_db_id);

/**
 * @brief Removes the to-do item from the calendar database.
 *
 * @param[in]   todo_id  The to-do ID to delete
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 * @see calendar_todo_insert_to_db()
 * @see calendar_todo_update_to_db()
 */
int calendar_todo_delete_from_db(int todo_id);

/**
 * @brief Updates the to-do item on the calendar database.
 * @details calendar_todo_get_db_id() is called internally to update the to-do item in the calendar database.
 *
 * @param[in]   todo		The calendar to-do handle 
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @see calendar_todo_insert_to_db()
 * @see calendar_todo_delete_from_db()
 * @see calendar_todo_get_from_db()
 * @see calendar_todo_get_db_id()
 */
int calendar_todo_update_to_db(calendar_todo_h todo);

/**
 * @brief Gets the to-do item handle associated with the given to-do DB ID.
 * @details This function gets the new to-do handle with the given to-do database ID from the calendar database. 
 * A new calendar to-do handle will be created. 
 *
 * @remarks The new to-do handle must be released with calendar_todo_destroy() by you. 
 *
 * @param[in]   todo_id			The to-do Database ID
 * @param[out]  todo			The calendar to-do handle 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE    Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY       Out of memory
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @see calendar_todo_destroy()
 * @see calendar_todo_get_db_id()
 */
int calendar_todo_get_from_db(int todo_id, calendar_todo_h *todo);

/**
 * @brief   Gets the database ID for the given calendar to-do handle. 
 *
 * @param[in]   todo			The calendar to-do handle
 * @param[out]  todo_db_id		The database ID of the to-do item (default : 0) \n
 *  0 means the to-do is not in calendar database
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_insert_to_db()
 * @see calendar_todo_delete_from_db()
 */
int calendar_todo_get_db_id(calendar_todo_h todo, int *todo_db_id);

/**
 * @brief   Gets the calendar database ID associated with the given calendar to-do handle. 
 *
 * @param[in]   todo			The calendar to-do handle
 * @param[out]  calendar_db_id	The calendar database ID fetched from the calendar to-do handle (default : 0)  \n
 * 0 means the to-do is not in calendar database. 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_insert_to_db()
 */
int calendar_todo_get_calendar_db_id(calendar_todo_h todo, int *calendar_db_id);

/**
 * @brief Gets the subject of the to-do from the calendar to-do handle.
 *
 * @remarks @a subject must be released with free() by you. 
 *
 * @param [in] todo				The calendar to-do handle 
 * @param [out] subject			The subject of the to-do \n If the subject does not exist, it is NULL
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_todo_set_subject()
 */
int calendar_todo_get_subject(calendar_todo_h todo, char **subject);

/**
 * @brief Sets a subject of the to-do handle.
 *
 * @param [in] todo		The calendar to-do handle 
 * @param [in] subject	The subject of the to-do
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_todo_get_subject()
 */
int calendar_todo_set_subject(calendar_todo_h todo, const char *subject);

/**
 * @brief Gets description of the to-do from the calendar to-do handle.
 *
 * @remarks @a description must be released with free() by you. 
 *
 * @param [in] todo				The calendar to-do handle 
 * @param [out] description		The description of the to-do \n If the description does not exist, it is NULL
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_todo_set_description()
 */
int calendar_todo_get_description(calendar_todo_h todo, char **description);

/**
 * @brief Sets a description of the to-do in the calendar to-do handle.
 *
 * @param [in] todo				The calendar to-do handle 
 * @param [in] description		The description of the to-do
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_todo_get_description()
 *
 */
int calendar_todo_set_description(calendar_todo_h todo, const char *description);

/**
 * @brief   Gets the priority for the given calendar to-do handle. 
 *
 * @param[in]   todo	   		The calendar to-do handle
 * @param[out]  priority			The priority of the to-do
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_set_priority()
 */
int calendar_todo_get_priority(calendar_todo_h todo, calendar_todo_priority_e *priority);

/**
 * @brief   Sets a priority for the given calendar to-do handle. 
 *
 * @param[in]  todo   		The calendar to-do handle
 * @param[in]  priority		The priority of the to-do
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_get_priority()
 */
int calendar_todo_set_priority(calendar_todo_h todo, calendar_todo_priority_e priority);

/**
 * @brief   Gets the status for the given calendar to-do handle. 
 *
 * @param[in]   todo	   		The calendar to-do handle
 * @param[out]  status			The status of the to-do
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_set_status() 
 */
int calendar_todo_get_status(calendar_todo_h todo, calendar_todo_status_e *status);

/**
 * @brief   Sets a status for the given calendar to-do handle. 
 *
 * @param[in]   todo   		The calendar to-do handle
 * @param[in]  status		The status of the to-do
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_get_status()
 */
int calendar_todo_set_status(calendar_todo_h todo, calendar_todo_status_e status);

/**
 * @brief   Gets the visibility for the given calendar to-do handle. 
 *
 * @param[in]   todo	   		The calendar to-do handle
 * @param[out]  visibility		The visibility of the to-do \n (default : #CALENDAR_VISIBILITY_PUBLIC)
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_set_visibility()
 */
int calendar_todo_get_visibility(calendar_todo_h todo, calendar_visibility_e *visibility);

/**
 * @brief   Sets a visibility for the given calendar to-do handle. 
 *
 * @param[in]   todo   		The calendar to-do handle
 * @param[in]  visibility	The visibility of the to-do
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_get_visibility()
 */
int calendar_todo_set_visibility(calendar_todo_h todo, calendar_visibility_e visibility);

/**
 * @brief Gets the location of the to-do from the calendar to-do handle.
 *
 * @remarks @a location must be released with free() by you. 
 *
 * @param [in] todo The calendar to-do handle 
 * @param [out] location The location of the to-do \n If the location does not exist, it is NULL
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_todo_set_location()
 *
 */
int calendar_todo_get_location(calendar_todo_h todo, char **location);

/**
 * @brief Sets a location of the to-do in the calendar to-do handle.
 *
 * @param [in] todo The calendar to-do handle 
 * @param [in] location The location of the to-do
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_todo_get_location()
 */
int calendar_todo_set_location(calendar_todo_h todo, const char *location);

/**
 * @brief	Gets the last modified time of the to-do item.
 *
 * @param[in]   todo       The calendar to-do handle 
 * @param[out]  modified_time The last modified time
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE				Successful
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY       Out of memory
 * @retval  #CALENDAR_ERROR_NO_DATA             Data not found
 *
 * @see calendar_todo_insert_to_db()
 * @see calendar_todo_update_to_db()
 *
 */
int calendar_todo_get_last_modified_time(calendar_todo_h todo, struct tm *modified_time); 
/**
 * @brief   Gets the timezone details for the given calendar to-do handle. 
 * @remarks @a timezone_name must be released with free() by you.
 *
 * @param[in]   todo   					The calendar to-do handle
 * @param[out]  timezone_name			The time zone name eg. Europe/Paris
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_set_timezone()
 * @see calendar_foreach_timezone()
 */
int calendar_todo_get_timezone(calendar_todo_h todo, char** timezone_name); 

/**
 * @brief   Sets the timezone for the given calendar to-do handle. 
 *
 * @param[in]   todo   					The calendar to-do handle
 * @param[in]  timezone_name			The timezone name eg. Europe/Paris
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_get_timezone()
 * @see calendar_foreach_timezone()
 */
int calendar_todo_set_timezone(calendar_todo_h todo, const char* timezone_name); 

/**
 * @brief Gets the start time of the to-do from the calendar to-do handle.
 *
 * @param [in] todo The calendar to-do handle 
 * @param [out] start_time The start date of the to-do
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #CALENDAR_ERROR_NO_DATA Data not found
 *
 * @see calendar_todo_set_start_time()
 */
int calendar_todo_get_start_time(calendar_todo_h todo, struct tm *start_time);

/**
 * @brief Sets a start time of the to-do in the calendar to-do handle. 
 *
 * @param [in] todo The calendar todo handle 
 * @param [in] start_time The start date of the todo
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see calendar_todo_get_start_time()
 *
 */
int calendar_todo_set_start_time(calendar_todo_h todo, struct tm start_time);


/**
 * @brief Gets the due time of the to-do from the calendar to-do handle.
 *
 * @param [in] todo The calendar to-do handle 
 * @param [out] due_time The due date of the to-do
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE					Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER	Invalid parameter
 * @retval #CALENDAR_ERROR_NO_DATA				Data not found
 *
 * @see calendar_todo_set_due_time()
 *
 */
int calendar_todo_get_due_time(calendar_todo_h todo, struct tm *due_time);

/**
 * @brief Sets a due time of the to-do in the calendar to-do handle.
 *
 * @param[in]   todo The calendar to-do handle 
 * @param[in]   due_time    The due date of the to-do
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_todo_get_due_time()
 */
int calendar_todo_set_due_time(calendar_todo_h todo, struct tm due_time);

/**
 * @brief  Retrieves all to-do items by invoking the given callback function iteratively. 
 *
 * @param[in]   callback    The callback function to invoke 
 * @param[in]   user_data   The user data to be passed to the callback function 
 *
 * @return   0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_todo_cb().
 *
 * @see calendar_foreach_query_todo_cb()
 */
int calendar_foreach_todo_from_db(calendar_foreach_query_todo_cb callback, void *user_data);

/**
 * @brief  Retrieves all to-do items with the given calendar database ID by invoking the given callback function iteratively. 
 *
 * @param[in]   callback    The callback function to invoke 
 * @param[in]   calendar_db_id      The calendar database ID to filter \n
 *                       1 means the default calendar on the device
 * @param[in]   user_data   The user data to be passed to the callback function 
 *
 * @return   0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_DB_FAILED No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_todo_cb().
 *
 * @see calendar_foreach_query_todo_cb()
 */
int calendar_query_todo_by_calendar(calendar_foreach_query_todo_cb callback, int calendar_db_id, void *user_data);

/**
 * @}
 */

/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_MULTIPLECALENDAR_MODULE
 * @{
 */

/**
 * @brief Gets the calendar from the calendar database.
 * @details This function gets the new calendar handle with the given calendar database ID from the calendar database. \n
 * The new calendar handle will be created. 
 *
 * @remarks The created calendar handle must be released with calendar_destroy() by you. 
 *
 * @param[in]   calendar_db_id		The calendar database ID
 * @param[out]  calendar			The calendar handle associated with the calendar database ID
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE  Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval  #CALENDAR_ERROR_OUT_OF_MEMORY       Out of memory
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @see calendar_destroy()
 */
int calendar_get_from_db(int calendar_db_id, calendar_h *calendar);

/**
 * @brief Destroys the calendar handle and releases all its resources.
 *
 * @param[out]  calendar   The calendar handle 
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_get_from_db()
 *
 */
int calendar_destroy(calendar_h calendar);

/**
 * @brief   Gets the database ID associated with the given calendar handle. 
 *
 * @param[in]   calendar		The calendar handle
 * @param[out]  calendar_db_id	The calendar database ID fetched from the calendar handle
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 *
 * @see calendar_get_from_db()
 */
int calendar_get_db_id(calendar_h calendar, int *calendar_db_id);

/**
 * @brief Gets the status whether the calendar is default calendar is or not in the device.
 *
 * @param [in] calendar			The calendar handle 
 * @param [out] is_default		@c true if the calendar is the default calendar in the device @c false \n
 * (default : @c false)
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 */
int calendar_get_is_default(calendar_h calendar, bool *is_default);

/**
 * @brief Gets the name of the calendar from the calendar handle.
 *
 * @remarks @a calendar_name must be released with free() by you. 
 *
 * @param [in] calendar			The calendar handle 
 * @param [out] calendar_name	The name of the calendar \n If the name does not exist, it is NULL
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 */
int calendar_get_name(calendar_h calendar, char **calendar_name);

/**
 * @brief Gets the visible status of the calendar handle.
 *
 * @param [in] calendar The calendar handle 
 * @param [out] is_visible @c true if the calendar is visible in the device, otherwise @c false \n
 * (default : @c false)
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 */
int calendar_get_is_visibile(calendar_h calendar, bool *is_visible);

/**
 * @brief Gets the color of the calendar from the calendar handle.
 *
 * @param [in] calendar The calendar handle 
 * @param [out] red		The calendar color which represents red among RGB
 * @param [out] green	The calendar color which represents green among RGB
 * @param [out] blue	The calendar color which represents blue among RGB
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #CALENDAR_ERROR_NONE Successful
 * @retval #CALENDAR_ERROR_INVALID_PARAMETER Invalid parameter
 */
int calendar_get_color(calendar_h calendar, unsigned char *red, unsigned char *green, unsigned char *blue);

/**
 * @brief   Gets the account database ID associated with the given calendar handle. 
 *
 * @param[in]   calendar	The calendar handle
 * @param[out]  account_db_id	The account database ID(default : 0) \n
 * 0 means the calendar is not related to any accounts.
 *
 * @return  0 on success, otherwise a negative error value.
 * @retval  #CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 */
int calendar_get_account_db_id(calendar_h calendar, int *account_db_id);

/**
 * @brief  Retrieves all calendars by invoking the given callback function iteratively. 
 *
 * @param[in]   callback    The callback function to invoke
 * @param[in]   user_data   The user data to be passed to the callback function
 *
 * @return	0 on success, otherwise a negative error value.
 * @retval	#CALENDAR_ERROR_NONE                Successful
 * @retval  #CALENDAR_ERROR_INVALID_PARAMETER   Invalid parameter
 * @retval	#CALENDAR_ERROR_DB_FAILED			No access to database
 *
 * @pre     This function requires an open connection to the calendar service by calendar_connect().
 *
 * @post	This function invokes calendar_foreach_query_event_cb().
 *
 * @see calendar_foreach_query_calendar_cb()
 */
int calendar_foreach_calendar_from_db(calendar_foreach_query_calendar_cb callback, void *user_data);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif


#endif
