/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */


#ifndef __TIZEN_SOCIAL_CALENDAR_TYPES_H__
#define __TIZEN_SOCIAL_CALENDAR_TYPES_H__

#include <stdbool.h>
#include <tizen.h>
#include <time.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_MODULE
 * @{
 */

/**
 * @brief Enumerations for calendar error type.
 */
typedef enum
{
    CALENDAR_ERROR_NONE                 = TIZEN_ERROR_NONE,                   /**< Successful */
    CALENDAR_ERROR_DB_FAILED            = TIZEN_ERROR_SOCIAL_CLASS | 0x02,    /**< No access to database */
    CALENDAR_ERROR_OUT_OF_MEMORY        = TIZEN_ERROR_OUT_OF_MEMORY,          /**< Out of memory */
    CALENDAR_ERROR_INVALID_PARAMETER    = TIZEN_ERROR_INVALID_PARAMETER,      /**< Invalid parameter */
    CALENDAR_ERROR_NO_DATA              = TIZEN_ERROR_SOCIAL_CLASS | 0x03,    /**< Requested data does not exist */
    CALENDAR_ERROR_ITERATOR_END         = TIZEN_ERROR_SOCIAL_CLASS | 0x04,    /**< Iterator is on last position */
} calendar_error_e;

/**
 * @brief Enumerations for calendar visibility type.
 */
typedef enum
{
    CALENDAR_VISIBILITY_PUBLIC          = 0,	/**< Public visibility */
    CALENDAR_VISIBILITY_PRIVATE,				/**< Private visibility */
    CALENDAR_VISIBILITY_CONFIDENTIAL			/**< Confidential visibility */
} calendar_visibility_e;

/**
 * @brief Enumerations for calendar timezone details.
 */
typedef struct
{
	char*		timezone_name;					/**< Timezone name eg. Europe/Paris */
	char*		city_name;						/**< City name eg. Paris */
	char*		country_name;					/**< Country name eg. France */
	char*		time_offset;					/**< Time offset eg. GMT+1 */
	char*		timezone_detail_filepath;		/**< Timezone detail file path eg. /.../Europe/Paris */
}calendar_foreach_timezone_s ;

/**
 * @brief  Called once for each timezone supported on a device. 
 * 
 * @param[out]  timezone   The timezone information 
 * @param[in]   user_data  The user data passed from the foreach function
 *
 * @return  @c true to continue with the next iteration of the loop or return @c false to break out of the loop.
 *
 * @pre calendar_foreach_timezone() will invoke this callback. 
 *
 * @see calendar_foreach_timezone()
 */
typedef bool (*calendar_foreach_timezone_cb)(calendar_foreach_timezone_s *timezone, void *user_data);
/**
 * @}
 */

/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_ATTENDEE_MODULE
 * @{
 */

  /**
 * @brief The calendar attendee handle.
 * @see calendar_event_add_attendee()
 * @see calendar_event_remove_attendee()
 */
typedef struct calendar_attendee_s *calendar_attendee_h; /* cal_value */

 /**
 * @brief The event attendee iterator.
 * @see calendar_event_get_attendee_iterator()
 */
typedef struct calendar_list_s *calendar_attendee_iterator_h;

/**
 * @}
 */


 /**
 * @addtogroup CAPI_SOCIAL_CALENDAR_EVENT_MODULE
 * @{
 */

/**
 * @brief The calendar event handle.
 */
typedef struct calendar_event_s *calendar_event_h;

/**
 * @brief The calendar recurrence exception iterator handle.
 * @see calendar_event_get_recurrence_exception_iterator()
 */
typedef struct calendar_list_s *calendar_recurrence_exception_iterator_h;

/**
 * @brief  Reminder interval type of event such as minutes, hours, days, or etc. 
 */
typedef enum
{
    CALENDAR_REMINDER_NONE = -1,            /**< No reminder set */
    CALENDAR_REMINDER_TIME_UNIT_MINUTE = 0, /**< Reminder interval in minutes */
    CALENDAR_REMINDER_TIME_UNIT_HOUR,       /**< Reminder interval in hours */
    CALENDAR_REMINDER_TIME_UNIT_DAY,        /**< Reminder interval in days */
    CALENDAR_REMINDER_TIME_UNIT_WEEK,       /**< Reminder interval in weeks */
    CALENDAR_REMINDER_TIME_UNIT_MONTH,      /**< Reminder interval in months */
} calendar_reminder_interval_type_e;


/**
 * @brief Enumerations of the frequency of event recurrence.
 */
typedef enum
{
    CALENDAR_RECURRENCE_NONE,           /**< No recurrence event */
    CALENDAR_RECURRENCE_DAILY,          /**< A Event occurs every day */
    CALENDAR_RECURRENCE_WEEKLY,         /**< A Event occurs on the same day of every week \n According to week flag, the event will recurrence every days of week */
    CALENDAR_RECURRENCE_MONTHLY,        /**< A Event occurs on the same day of every month */
    CALENDAR_RECURRENCE_YEARLY,         /**< A Event occurs on the same day of every year */
} calendar_recurrence_frequency_e;

/**
 * @brief Enumerations of weekly recurrence event which days of week will be repeated.
 */
typedef enum
{
    CALENDAR_WEEK_FLAG_NONE = 0x00,     /**< No repetition */
    CALENDAR_WEEK_FLAG_SUN  = 0x01,     /**< Repeat event each Sunday */
    CALENDAR_WEEK_FLAG_MON  = 0x02,     /**< Repeat event each Monday */
    CALENDAR_WEEK_FLAG_TUE  = 0x04,     /**< Repeat event each Tuesday */
    CALENDAR_WEEK_FLAG_WED  = 0x08,     /**< Repeat event each Wednesday */
    CALENDAR_WEEK_FLAG_THU  = 0x10,     /**< Repeat event each Thursday */
    CALENDAR_WEEK_FLAG_FRI  = 0x20,     /**< Repeat event each Friday */
    CALENDAR_WEEK_FLAG_SAT  = 0x40,     /**< Repeat event each Saturday */
    CALENDAR_WEEK_FLAG_ALL  = 0xFF      /**< Repeat event every day */
} calendar_week_flag_e;

/**
 * @brief  Called once for each event returned from the query.
 * 
 * @param[in]   event	The calendar event handle 
 * @param[in]   user_data	The user data passed from the foreach function 
 *
 * @return  @c true to continue with the next iteration of the loop or return @c false to break out of the loop.
 *
 * @pre	calendar_foreach_event_from_db() will invoke this callback. 
 * @pre	calendar_query_event_by_subject() will invoke this callback. 
 * @pre	calendar_query_event_by_description() will invoke this callback. 
 * @pre	calendar_query_event_by_location() will invoke this callback. 
 * @pre	calendar_query_event_by_period() will invoke this callback. 
 * @pre	calendar_query_event_by_time_last_modified() will invoke this callback.
 *
 * @see	calendar_query_event_by_subject()
 * @see	calendar_query_event_by_description()
 * @see	calendar_query_event_by_location()
 * @see	calendar_query_event_by_period()
 * @see	calendar_foreach_event_from_db()
 */
typedef bool (*calendar_foreach_query_event_cb)(calendar_event_h event, void *user_data);

/**
 * @}
 */

/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_MODULE
 * @{
 */

/**
 * @brief       Called when any database changes has occurred. 
 *
 * @param[in]   user_data	The user data passed from the callback registration function
 *
 * @pre calendar_set_db_changed_cb() will invoke this callback. 
 * @pre calendar_event_insert_to_db(), calendar_event_delete_from_db() or calendar_event_update_to_db() will invoke this callback.
 *
 * @see calendar_set_db_changed_cb()
 * @see calendar_unset_db_changed_cb()
 * @see calendar_event_insert_to_db()
 * @see calendar_event_delete_from_db()
 * @see calendar_event_update_to_db()
 */
typedef void (*calendar_db_changed_cb)(void *user_data);

/**
 * @}
 */

/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_TODO_MODULE
 * @{
 */

/**
 * @brief The calendar to-do handle.
 */
typedef struct calendar_event_s *calendar_todo_h;

/**
 * @brief Calendar to-do item priority
 */
typedef enum
{
    CALENDAR_TODO_PRIORITY_LOW          = 0,	/**< Low priority */
    CALENDAR_TODO_PRIORITY_NORMAL,				/**< Normal priority */
    CALENDAR_TODO_PRIORITY_HIGH					/**< High priority */
} calendar_todo_priority_e;

typedef enum
{
    CALENDAR_TODO_STATUS_NONE          = 0,		/**< None */
    CALENDAR_TODO_STATUS_NEEDS_ACTION,			/**< Needs action status */
    CALENDAR_TODO_STATUS_COMPLETED,				/**< Completed status */
    CALENDAR_TODO_STATUS_IN_PROCESS,			/**< Work in process status */
    CALENDAR_TODO_STATUS_CANCELED				/**< Canceled status */
} calendar_todo_status_e;

/**
 * @brief  Called once for each calendar to-do item. 
 * 
 * @param[in]   todo       The calendar to-do handle 
 * @param[in]   user_data  The user data passed from the foreach function
 *
 * @return  @c true to continue with the next iteration of the loop or return @c false to break out of the loop.
 *
 * @pre calendar_foreach_todo_from_db() will invoke this callback.
 * @see calendar_foreach_todo_from_db()
 */
typedef bool (*calendar_foreach_query_todo_cb)(calendar_todo_h todo, void *user_data);
/**
 * @}
 */

/**
 * @ingroup     CAPI_SOCIAL_CALENDAR_MODULE
 * @defgroup    CAPI_SOCIAL_CALENDAR_MULTIPLECALENDAR_MODULE Multiple Calendar
 *
 * @brief Calendar definition and Calendar related operations.
 *
 * @section CAPI_SOCIAL_CALENDAR_MULTIPLECALENDAR_MODULE_HEADER Required Header
 *   \#include <calendar.h>
 *
 * @section CAPI_SOCIAL_CALENDAR_MULTIPLECALENDAR_MODULE_OVERVIEW Overview
 * Calendar
 *
 * <BR>
 */

/**
 * @addtogroup CAPI_SOCIAL_CALENDAR_MULTIPLECALENDAR_MODULE
 * @{
 */

/**
 * @brief The calendar handle.
 */
typedef struct calendar_event_s *calendar_h;

/**
 * @brief Definition for default calendar database ID
 */
#define DEFAULT_CALENDAR_DB_ID			1

/**
 * @brief   The structure of calendar in search results.
 *
 * @see     calendar_foreach_query_calendar_cb()
 */
typedef struct {
	int     calendar_db_id;		    		/**< calendar database ID */
	char*   calendar_name;				    /**< calendar name */
	bool    calendar_is_default;		    /**< default calendar status */
	bool    calendar_is_visible;		    /**< calendar visible status */
    unsigned char calendar_color_red;       /**< calendar color red among RGB */
    unsigned char calendar_color_green;       /**< calendar color green among RGB */
    unsigned char calendar_color_blue;       /**< calendar color blue among RGB */
	int     account_db_id;					/**< account database ID */
}calendar_query_calendar_s;

/**
 * @brief       The callback function to get the calendar
 *
 * @param[in]   calendar            The calendar
 * @param[in]   user_data           The user data passed from the foreach function
 *
 * @return  @c true to continue with the next iteration of the loop or @c false to break out of the loop.
 *
 * @pre calendar_foreach_calendar_from_db() will invoke this callback.
 *
 * @see calendar_foreach_calendar_from_db()
 */
typedef bool (*calendar_foreach_query_calendar_cb)(calendar_query_calendar_s *calendar, void *user_data);

/**
 * @}
 */


#ifdef __cplusplus
}
#endif

#endif  /* __TIZEN_SOCIAL_CALENDAR_TYPES_H__ */
